# Generated by Django 4.2.7 on 2023-11-07 12:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user_visit", "0003_uservisit_context"),
    ]

    operations = [
        migrations.AddField(
            model_name="uservisit",
            name="browser",
            field=models.CharField(blank=True, default="", max_length=200),
        ),
        migrations.AddField(
            model_name="uservisit",
            name="device",
            field=models.CharField(
                blank=True, default="", max_length=200, verbose_name="Device type"
            ),
        ),
        migrations.AddField(
            model_name="uservisit",
            name="os",
            field=models.CharField(
                blank=True, default="", max_length=200, verbose_name="Operating System"
            ),
        ),
    ]
