# Generated by Django 3.2.12 on 2022-03-07 17:02

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user_visit", "0002_add_created_at"),
    ]

    operations = [
        migrations.AddField(
            model_name="uservisit",
            name="context",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                help_text="Used for storing ad hoc / ephemeral data - e.g. GeoIP.",
                null=True,
            ),
        ),
    ]
